import java.net.Socket;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.IOException;

/**
 * Ein einfacher Clientantwortprozess fr den Chatserver.
 * 
 * @author Bernd Gramlich 
 * @version 2014-12-09
 */
public class ClientAntwortProzess implements Runnable
{
    private Socket meinSocket;
    private ServerProzess server;
    private String meinNick;
    
    private PrintWriter gibAnClient;
    private BufferedReader liesVomClient;

    /**
     * Konstruktor.
     * Erzeuge einen neuen Clientprozess am Socket s fr
     * den Serverprozess p.
     */
    public ClientAntwortProzess (Socket s, ServerProzess p)
    {
        meinSocket = s;
        server     = p;
        meinNick   = null;
    }

    /**
     * Ablaufplan.
     */
    public void run()
    {
        try
        {
            /*
             * Datenstrme an den und vom Client.
             */
            gibAnClient   = new PrintWriter (meinSocket.getOutputStream (), true);
            liesVomClient = new BufferedReader (new InputStreamReader (meinSocket.getInputStream ()));

            /*
             * Zustndiges Protokoll.
             */
            Serverprotokoll protokoll = new Serverprotokoll ();

            /*
             * Ein- und Ausgabezeile und die Adressatin.
             */
            String eingabeZeile;
            String ausgabeZeile;
            String adressatin;

            /*
             * Aktivitt des Clients.
             */
            boolean aktiv = true;

            /*
             * Reagiere zunchst auf die leere Eingabe vom Client.
             */
            ausgabeZeile = protokoll.verarbeite (null);
            gibAnClient.println (ausgabeZeile);

            /*
             * Chatte gem Protokoll, solange der Client sendet.
             */
            eingabeZeile = liesVomClient.readLine ();

            if (eingabeZeile == null)
            {
                aktiv = false;
            }

            while (aktiv)
            {
                ausgabeZeile = protokoll.verarbeite (eingabeZeile);
                adressatin   = protokoll.gibAdressatin ();

                if (ausgabeZeile.equals ("Bye."))
                {
                    aktiv = false;
                    break;
                }
                else if (ausgabeZeile.startsWith ("nick "))
                {
                    String neuerNick = ausgabeZeile.substring (5);
                    boolean nickaenderungErfolgreich = server.aendereNick (meinNick, neuerNick, this);
                    
                    if (nickaenderungErfolgreich)
                    {
                        meinNick = neuerNick;
                        protokoll.vermerkeNick ();
                    }
                }
                else if (ausgabeZeile.equals ("who"))
                {
                    server.nenneNicks (meinNick, "");
                }
                else if (ausgabeZeile.startsWith ("who "))
                {
                    server.nenneNicks (meinNick, ausgabeZeile.substring (4));
                }
                else if (null == adressatin)
                {
                    server.sendeAnAlle (meinNick + ": " + ausgabeZeile);
                }
                else if (adressatin.equals (""))
                {
                    gibAnClient.println (ausgabeZeile);
                }
                else
                {
                    server.sendeAn (adressatin, meinNick + ausgabeZeile, this);
                }

                eingabeZeile = liesVomClient.readLine ();

                if (eingabeZeile == null)
                {
                    aktiv = false;
                }
            }
            
            /*
             * Melde Dich vom Server ab.
             */
            server.meldeAb (meinNick);
            
            /*
             * Schliee die Datenstrme und den Socket.
             */
            gibAnClient.close ();
            liesVomClient.close ();
            meinSocket.close();
        }
        catch (IOException fehler)
        {
            fehler.printStackTrace();
        }
    }
    
    /**
     * Sende die angegebene Nachricht an den Client.
     */
    public synchronized boolean sende (String nachricht)
    {
        if (gibAnClient != null)
        {
            gibAnClient.println (nachricht);
            
            return true;
        }
        else
        {
            return false;
        }
    }
    
    /**
     * Nenne meine Adresse im Netz.
     */
    public String gibAdresse ()
    {
        return meinSocket.getInetAddress ().getCanonicalHostName () + " ("
           + meinSocket.getInetAddress ().getHostAddress () + ")";
    }
}