import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Collections;
import java.util.Hashtable;

/**
 * Der Serverprozess fr den Chatserver.
 * 
 * @author Bernd Gramlich 
 * @version 2014-12-10
 */
public class ServerProzess implements Runnable
{
    /*
     * Der ServerSocket, auf dem ich laufe.
     */
    private ServerSocket serverSocket;
    
    /*
     * Eine Tabelle, die zu jedem Nick (Anmeldenamen) den
     * zugehrigen Clientantwortprozess enthlt.
     */
    private Hashtable <String, ClientAntwortProzess> prozess;

    /**
     * Konstruktor.
     * Erzeuge einen Serverprozess auf dem angegebenen ServerSocket.
     */
    public ServerProzess (ServerSocket s)
    {
        serverSocket = s;
        
        prozess = new Hashtable <String, ClientAntwortProzess> ();
    }

    /**
     * Ablaufplan.
     * Warte auf neue Clientverbindungen. Starte dabei fr jeden
     * sich anmeldenden Client einen neuen ClientAntwortProzess.
     */
    public void run ()
    {
        try
        {
            while (true)
            {
                Socket clientSocket = serverSocket.accept ();
                new Thread (new ClientAntwortProzess (clientSocket, this)).start();
            }
        }
        catch (IOException e)
        {
            System.err.println ("Serverprozess scheitert.");
            System.exit (1);
        }
    }
    
    /**
     * Sende die angegebene Nachricht an alle.
     */
    public void sendeAnAlle (String nachricht)
    {
        for (ClientAntwortProzess c: prozess.values ())
        {
            c.sende (nachricht);
        }
    }
    
    /**
     * Sende die angegebene Nachricht an die angegebene Person.
     */
    public boolean sendeAn (String name, String nachricht, ClientAntwortProzess absender)
    {
        if (prozess.containsKey (name))
        {
            ClientAntwortProzess empfaenger = prozess.get (name);
            empfaenger.sende (nachricht);
            
            return true;
        }
        else
        {
            absender.sende ("There's no one named " + name + ".");
            
            return false;
        }
    }
    
    /**
     * Erstelle oder ndere den Nick eines Clientantwortprozesses.
     */
    public synchronized boolean aendereNick (String alterName, String neuerName, ClientAntwortProzess absender)
    {
        if (prozess.containsKey (neuerName))
        {
            absender.sende ("Somebody is already called " + neuerName + ".");
            return false;
        }
        else
        {
            if (alterName != null)
            {
                prozess.remove (alterName);
                sendeAnAlle (alterName + " will henceforth be known as " + neuerName + ".");
            }
            else
            {
                sendeAnAlle (neuerName + " has joined.");
            }   
            
            prozess.put (neuerName, absender);
            
            return true;
        }
    }
    
    /**
     * Melde den angegebenen Nick und seinen Clientantwortprozess ab.
     */
    public synchronized void meldeAb (String name)
    {
        prozess.remove (name);
        sendeAnAlle (name + " has left.");
    }
    
    /**
     * Teile der Benutzerin name alle Nicks und ihre Herkunft mit,
     * deren Nick mit muster beginnt.
     */
    public synchronized void nenneNicks (String name, String muster)
    {
        for (String nutzer: Collections.list (prozess.keys ()))
        {
            if (nutzer.startsWith (muster))
            {
                prozess.get (name).sende (nutzer + " is connected from " + prozess.get (nutzer).gibAdresse () + ".");
            }
        }
    }
}
